function  xx = estCavLength_cancelZero_fastSearch_forJont(varargin)
% clc
% -------------------------------------------------------------------------
% [len,dia,beta] = estCavLength_cancelZero_fastSearch(varargin)
% -------------------------------------------------------------------------
% Inputs (3):
% len0 - initial length estimates
% Pk - measured cavity pressures
% f - frequency (corresponding to elements of Pk)
%
% Inputs (1):
% filename - point to a 4cdata.mat file
%
% Ouputs:
% len - cavity lengths
% dia - cavity diameters
% beta - beta values
% -------------------------------------------------------------------------

if nargin == 0
    close all
    load 4cdata_MA17E_best.mat
    %load('C:\Users\Sarah\Desktop\JONTCAL\4cdata_MA17E_best.mat');
    %load('C:\Users\Sarah\Documents\WORK\Mimosa\SarahTests\ProbeEval\MA174F6\RevCCalibrationTestsMarch16\Calibration\Meas3V0.8_4Cav_13_18_23_29.6\4cdata.mat')
    %load('C:\Users\Sarah\Documents\WORK\Mimosa\SarahTests\ProbeEval\ER10C_2597\Calibration\Meas1\4cdata.mat')
    len0 = cal.len0;
    Pk=cavityResponse.';
    f=sig.FrequenciesAll;
    len4c = len;
elseif nargin == 1
    load(varargin{1}) % point to a calibration file
    len0 = cal.len0;
    Pk=cavityResponse.';
    f=sig.FrequenciesAll;
    len4c = len;
else
    len0 = varargin{1};
    Pk = varargin{2};
    f = varargin{3};
end

% -------------------------------------------------------------------------
% SETTINGS
% -------------------------------------------------------------------------
normP1 = 1; % normalize by the first cavity
complexPL =1; % path length - complex versus absValue of the ratio
cancelFunction = 1; % choose canceling fn: 0: P0 = 1+ R, 1: P0 = (1+R)/(1-R) NO DIFFERENCE?

% style for plot
summaryFig = 1;
showFigs = 0;
pauseFigs = showFigs; % if showFigs is set to 1, pause them enough to see
fs = 14; cc = 'kgrb';

% search parameters
nSearch = 2; % how many times do we "zoom in" to search
N = 5; % length range
M = 5; % diameter range

% physical constants (SRR: I think these have a lot of influence on beta00)
dblTemp = 20; %18; % degrees C
dblDT = (dblTemp-27); % degrees C, Keefe's values for acoustic params are referenced to 27C
rho0 = (1-0.00335*dblDT)*1.1769e-3*(100^3/1000); % kg/(m^3)
mu = (1+0.0025*dblDT)*1.846e-4*(100/1000); % kg/(s*m)
c0 = (1+0.00166*dblDT)*3.4723e4*(1/100); % m/s
beta0Factor = 1.045; % 1.085 for mason, 1.045 for keefe

% save plots
%location = 'C:\Users\Sarah\Documents\WORK\Mimosa\RevC_fromDon\files\4CCalibration\srr\twoCavCal\FindNull\MA17E\testIt\';
location = 'findNullFigs/';
% -------------------------------------------------------------------------

if exist(location,'dir'), rmdir(location,'s'); end
mkdir(location);

% save range
if size(Pk,2) > size(Pk,1), Pk = Pk.'; end

if showFigs ==1,
    % plot pressure responses
    h0 = figure(); set(gca,'fontsize',fs); hold on;
    for k1 = 1:min(size(Pk)), loglog(f/1000,abs(Pk(:,k1)),cc(k1),'linewidth',2); end
end

% initials lengths, null frequencies, null indices
len = NaN(1,min(size(Pk)));
dia = NaN(1,min(size(Pk)));
beta0 = NaN(1,min(size(Pk)));
err = NaN(1,min(size(Pk))); % possible error in path length minimization

% normalize by P1, or not (in which case, also fit null of P1)
if normP1 == 1, sk = 2; else sk = 1; end

if summaryFig ==1
    h5 = figure(); % set(gcf, 'Position', get(0,'Screensize'))
    nSubplots = min(size(Pk)) - (sk-1);
    subplotN = ceil(sqrt(nSubplots));
end

% -------------------------------------------------------------------------
% LOOP OVER THE CAVITIES
% -------------------------------------------------------------------------
for k1 = sk:min(size(Pk))
    
    % ---------------------------------------------------------------------
    % FIND AN INITIAL ESTIMATE FOR f0 and w0
    % ---------------------------------------------------------------------
    f0_init = (c0/(4*len0(k1)*10^-3)); % f0 from nominal cavity length, in Hz
    
    % fine-tune f0 estimate based on Pk data
    rangeSearch = .3; % increase if len0 is bad and nulls aren't being found
    [tmp,ind_fSearchMin]=min(abs(f-f0_init*(1-rangeSearch)));
    [tmp,ind_fSearchMax]=min(abs(f-f0_init*(1+rangeSearch)));
    [tmp,ind_f0_local] = min(abs(Pk(ind_fSearchMin:ind_fSearchMax,k1)));
    ind_f0 = ind_f0_local +ind_fSearchMin-1;
    f0 = f(ind_f0); %in Hz
    w0 = 2*pi*f0; % in radians
    
    if showFigs ==1,
        figure(h0) % mark the original estimate for the null frequency
        plot(f0/1000,abs(Pk(ind_f0,k1)),[cc(k1),'s'],'linewidth',2);
    end
    
    % ---------------------------------------------------------------------
    % MINIMIZATION RANGE:
    % Initialize the range over which path-length minimization will occur
    % ---------------------------------------------------------------------
    rangeSearch = 0.05;
    [tmp,ind_fSearchMin]=min(abs(f-f0*(1-rangeSearch))); % this makes the ind range bigger at high freqs
    [tmp,ind_fSearchMax]=min(abs(f-f0*(1+rangeSearch)));
    Pk_search = Pk(ind_fSearchMin:ind_fSearchMax,k1).';
    f_search = f(ind_fSearchMin:ind_fSearchMax);
    w_search = 2*pi*f_search; s_search = 0+1j*w_search;
    P1_search = Pk(ind_fSearchMin:ind_fSearchMax,1).';
    
    % search function - divide by shortest cavity, if applicable
    if normP1 == 1, Pk_search = Pk_search./P1_search; end
    
    % ---------------------------------------------------------------------
    % INITIALIZE DAMPING
    % We will make our beginning assumption based on the "known" cavity
    % diameter. Previously, I have done this by matching null height, then
    % matching the damping.
    %
    % FUNCTIONAL FORM: 2LK = j(a*w+d*sqrt(w)) + d*sqrt(w)
    % a - proportional to cavity length
    % d - dependent on a, and inversely proportional to cavity diameter
    % ---------------------------------------------------------------------
    
    % find b0 based on the measured diameter
    dia0 = 7.5; beta00 = 1000*beta0Factor*sqrt(8*mu/rho0)/dia0;
    b0 = sqrt(2)/beta00;
    a0 = pi/(w0+(1/b0)*sqrt(w0));
    G0 = exp(-1j*(a0*2*pi*f_search+(a0/b0)*sqrt(2*pi*f_search)) - (a0/b0)*sqrt(2*pi*f_search));
    if cancelFunction == 1, P0 = 2*(1+G0)./(1-G0); else P0 = 1+G0; end
    
    % normalize initial estimate
    ratio = Pk_search./P0; %sum(abs(diff(abs(ratio_min))))
    kScale = mean(abs(ratio));
    P0 = P0*kScale; % scale to match Pk null in height
    
    %  OLD METHOD to find damping
    %     mag_p0 = abs(Pk_search(ind_f0-ind_fSearchMin)); %abs(Pk(ind_f0,k1)); % null height
    %     d0 = -(1/sqrt(w0))*log(-mag_p0+1);
    %     a0 = (pi - d0*sqrt(w0))/w0;
    %     G0 = exp(-1j*(a0*2*pi*f_search+d0*sqrt(2*pi*f_search)) - d0*sqrt(2*pi*f_search));
    %     if cancelFunction == 1, P0 = 2*(1+G0)./(1-G0); else P0 = 1+G0; end
    %
    %     h3 = figure(); hold on; set(gca,'fontsize',fs); % local figure
    %     title({'Find intial coefficients for \kappa';'a=2L_k/c and d=2^{1/2}L_k \beta_0/c'})
    %     ylabel(['P_',num2str(k1),' and Zero-Canceling Function P_z']); xlabel('Frequency [kHz]')
    %     plot(f_search/1000,abs(Pk_search),'b');
    %     plot(f_search/1000,abs(P0),'m--');
    %
    %     sDelta = sign(sum(abs(Pk_search)-abs(P0))); % is estimate above or below data?
    %     while sign(sum(abs(Pk_search)-abs(P0))) == sDelta
    %         d0 = (1-sDelta*0.05)*d0; a0 = (pi-d0*sqrt(w0))/w0;
    %         P0= 1+exp(-1j*(a0*2*pi*f_search+d0*sqrt(2*pi*f_search)) - d0*sqrt(2*pi*f_search));
    %         if cancelFunction==1
    %             kScale = mag_p0*((1+exp(-d0*sqrt(w0))))/(1-exp(-d0*sqrt(w0)));
    %         else kScale = mag_p0/(1-exp(-d0*sqrt(w0)));
    %         end
    %         P0 = P0*kScale; plot(f_search/1000,abs(P0),'m--');
    %     end
    %     b0 = a0/d0; % actually we want to start at 7.5
    
    % ---------------------------------------------------------------------
    % INITIALIZE PLOTS
    % ---------------------------------------------------------------------
    if showFigs ==1,
        % initial and final estimates
        h1 = figure(); PP = get(h1,'position'); PP(3) = 1.5*PP(3); PP(1) = PP(1)/2; set(h1,'position',PP);
        subplot(121); set(gca,'fontsize',fs); hold on; % original P complex plane
        plot(real(Pk_search),imag(Pk_search),'k.','markersize',14);
        plot(real(P0),imag(P0),'r--','linewidth',2);
        title('Complex Pressure at Null');
        xlabel(['Real Part of P_',num2str(k1),' and P_z']); ylabel(['Imaginary Part of P_',num2str(k1),' and P_z'])
        subplot(122); set(gca,'fontsize',fs); hold on % original P mag v f
        plot(f_search/1000,abs(Pk_search),'k.','markersize',14);
        plot(f_search/1000,abs(P0),'r--','linewidth',2);
        title('Pressure Magnitude at Null');
        xlabel('Frequency [kHz]'); ylabel(['|P_',num2str(k1),'| and |P_z|'])
        
        % show path-length minimization
        h2 = figure(); PP = get(h2,'position'); PP(3) = 1.5*PP(3); PP(1) = PP(1)/2; set(h2,'position',PP);
        subplot(121); set(gca,'fontsize',14); hold on; % ratio P complex plane
        title('Complex Path Minimization Ratio')
        xlabel(['Real Part of P_',num2str(k1),'/P_z']); ylabel(['Imaginary Part of P_',num2str(k1),'/P_z'])
        subplot(122); set(gca,'fontsize',14); hold on % ratio P mag v f
        title('Magnitude Path Minimization Ratio')
        xlabel('Frequency [kHz]'); ylabel(['|P_',num2str(k1),'/P_z|'])
        
        h4 = figure();
    end % if showFigs
    
    % ---------------------------------------------------------------------
    % FIND MINIMIZING PARAMETERS
    % ---------------------------------------------------------------------
    % iterate around initial estimates
    a_it = linspace(a0*.98,a0*1.02,N);
    b_it = linspace(b0*.1,b0*1.9,M);
    
    % initialize path-length matrix
    length_sums = NaN(N,M);
    
    minCounter = 0;
    
    while minCounter < nSearch
        
        for k2 = 1:N % a (length)
            for k3 = 1:M % b (diameter)
                
                % calculate test function
                G = exp(-1j*(a_it(k2)*w_search+(a_it(k2)/b_it(k3))*sqrt(w_search)) - (a_it(k2)/b_it(k3))*sqrt(w_search));
                if cancelFunction == 1, P = 2*(1+G)./(1-G); else P = 1+G; end
                
                %ratio = Pk_search./P;
                ratio = (Pk_search/min(abs(Pk_search)))./(P/min(abs(P)));
                
                % calculate path length
                if complexPL ==1, length_sums(k2,k3) = sum(abs(diff((ratio))));
                else length_sums(k2,k3) = sum(abs(diff(abs(ratio)))); end
                
                if showFigs ==1,
                    figure(h2)
                    subplot(121); plot(real(ratio),imag(ratio),'color',[.5 .5 .5])
                    subplot(122); plot(f_search/1000,abs(ratio),'color',[.5 .5 .5])
                end
            end
        end
        
        % find minimum path length
        [tmp,ind] = min(length_sums(:));
        a_ind = rem(ind-1,N)+1;
        b_ind= floor((ind-1)/N)+1;
        
        % plot current search
        if showFigs ==1,
            figure(h4)
            imagesc(log(length_sums)); hold on; colormap gray
            plot(b_ind,a_ind,'rp','markersize',16,'linewidth',2); hold off;
            if pauseFigs ==1, pause(2); end
        end
        
        % adjust search range
        if a_ind >1 && a_ind < length(a_it), a_it = linspace(a_it(a_ind-1),a_it(a_ind+1),N);
        elseif a_ind ==1, a_it = linspace(a_it(1)*.9,a_it(2),N);
            if minCounter == 0, err(k1) = 1; end
        else a_it = linspace(a_it(end-1),a_it(end)*1.1,N);
            if minCounter == 0, err(k1) = 1; end
        end
        if b_ind >1 && b_ind < length(b_it), b_it = linspace(b_it(b_ind-1),b_it(b_ind+1),M);
        elseif b_ind ==1, b_it = linspace(b_it(1),b_it(3),M);
            if minCounter == 0, err(k1) = 1; end
        else b_it = linspace(b_it(end-2),b_it(end),M);
            if minCounter == 0, err(k1) = 1; end
        end
        
        if a_ind >1 && a_ind < length(a_it) && b_ind >1 && b_ind < length(b_it)
            err(k1) = 0; %min inside boundaries, at any point
        end
        
        minCounter = minCounter+1;
    end
    
    a0 = a_it(a_ind); b0 = b_it(b_ind);
    
    % find scale factor
    G = exp(-1j*(a0*w_search+(a0/b0)*sqrt(w_search)) - (a0/b0)*sqrt(w_search));
    if cancelFunction == 1, P = 2*(1+G)./(1-G); else P = 1+G; end
    ratio_min = Pk_search./P; %sum(abs(diff(abs(ratio_min))))
    kScale = mean(abs(ratio_min));
    P = P*kScale; % scale to match Pk null in height
    
    if showFigs ==1,
        figure(h4); set(gca,'fontsize',fs)% show minimization matrix
        imagesc(log(length_sums)); hold on; colormap gray
        xlabel('Vary d=2^{1/2}L_k\beta_0/c'); ylabel('Vary a=2L_k/c');
        title(['Search for shortest path length of P_',num2str(k1),'/P_z Ratio'])
        plot(b_ind,a_ind,'rp','markersize',16,'linewidth',2);
        
        figure(h1)
        subplot(121);
        plot(real(P),imag(P),'g','linewidth',2);
        subplot(122);
        plot(f_search/1000,abs(P),'g','linewidth',2);
        lgd = legend(['P_',num2str(k1),' data'],'P_z initial','P_z final');
        set(lgd,'fontsize',fs-2,'location','north')
        
        figure(h2)
        subplot(121)
        plot(real(ratio_min),imag(ratio_min),'g','linewidth',3)
        subplot(122)
        plot(f_search/1000,abs(ratio_min),'g','linewidth',3)
    end
    
    % ---------------------------------------------------------------------
    % DETERMINE LENGTH & DIAMETER
    % Find kappa, beta, dia, len, etc
    % a = 2L/c and d = L*beta00*sqrt(2)/c, d = a/b
    % ---------------------------------------------------------------------
    len(k1) = 1000*a0*c0/2; % mm
    beta0(k1) = sqrt(2)/b0;
    dia(k1) = 1000*beta0Factor*sqrt(8*mu/rho0)/beta0(k1); %mm
    
    % ---------------------------------------------------------------------
    % SAVE PLOTS
    % ---------------------------------------------------------------------
    if showFigs ==1,
        figure(h1)
        filename=[location, 'Cav',num2str(k1), 'Results'];
        set(gcf,'PaperPositionMode','auto')
        eval(char(strcat('print -depsc2 ',{' '},filename)))
        eval(char(strcat('print -dbitmap ',{' '},filename)))
        saveas(gcf,filename,'fig')
        
        figure(h2)
        filename=[location,'Cav',num2str(k1), 'PlotSearch'];
        set(gcf,'PaperPositionMode','auto')
        eval(char(strcat('print -depsc2 ',{' '},filename)))
        eval(char(strcat('print -dbitmap ',{' '},filename)))
        saveas(gcf,filename,'fig')
        
        %     figure(h3)
        %     filename=[location,'Cav',num2str(k1) 'StartingParams'];
        %     set(gcf,'PaperPositionMode','auto')
        %     eval(char(strcat('print -depsc2 ',{' '},filename)))
        %     eval(char(strcat('print -dbitmap ',{' '},filename)))
        %     saveas(gcf,filename,'fig')
        
        figure(h4)
        filename=[location,'Cav',num2str(k1),  'MatrixSearch'];
        set(gcf,'PaperPositionMode','auto')
        eval(char(strcat('print -depsc2 ',{' '},filename)))
        eval(char(strcat('print -dbitmap ',{' '},filename)))
        saveas(gcf,filename,'fig')
        
        %return
        close(h1); close(h2); %close(h3);
        close(h4);
    end % if showFigs
    
    if summaryFig ==1
        figure(h5); % set(gcf, 'Position', get(0,'Screensize'))
        subplot(subplotN,subplotN,k1-(sk-1)); set(gca,'fontsize',fs-2); hold on
        plot(f_search/1000,abs(Pk_search),'k.')
        if exist('len4c','var')
            a4C = 2*len4c(k1)*10^-3/c0;
            b4C = sqrt(2)/(1000*beta0Factor*sqrt(8*mu/rho0)/7.5);
            G4C = exp(-1j*(a4C*2*pi*f_search+(a4C/b4C)*sqrt(2*pi*f_search)) - (a4C/b4C)*sqrt(2*pi*f_search)); 
            if cancelFunction == 1, P4C = 2*(1+G4C)./(1-G4C); else P4C = 1+G4C; end
            %K4C = mean(abs(Pk_search./P4C)); P4C = K4C*P4C;
            K4C = min(abs(P))/min(abs(P4C)); P4C = K4C*P4C;
            plot(f_search/1000,abs(P4C),':','color',[.6 .6 .6]);
        end
        plot(f_search/1000,abs(P),'r-');
        xlimh = get(gca,'xlim'); ylimh = get(gca,'ylim');
        ylimh(2)= ylimh(2) + .5*diff(ylimh);
        set(gca,'ylim',ylimh);
        text(xlimh(1)+0.05*diff(xlimh),ylimh(2)-.05*diff(ylimh),['Length=',num2str(len(k1)),'mm'],'color','r')
        text(xlimh(1)+0.05*diff(xlimh),ylimh(2)-.15*diff(ylimh),['Diameter=',num2str(dia(k1)),'mm'],'color','r')
        text(xlimh(1)+0.05*diff(xlimh),ylimh(2)-.25*diff(ylimh),['\beta_0=',num2str(beta0(k1)),'mm'],'color','r')
        if err(k1) == 1,
            text(xlimh(1)+0.05*diff(xlimh),ylimh(2)-.35*diff(ylimh),'(Failed Minimization!)','color','r')
        end
        if k1-(sk-1) == 1
            lgd = legend('data','4C model','fit');
            set(lgd,'fontsize',8)
            set(lgd,'location','southeast')
            xlimh(2) = xlimh(2) + diff(xlimh);
            set(gca,'xlim',xlimh)
        end
    end
    
end % END LOOP OVER THE CAVITIES

% -------------------------------------------------------------------------
% FINALIZE CAVITY PRESSURES FIGURE
% -------------------------------------------------------------------------
if showFigs == 1,
    figure(h0)
    axis tight; set(gca,'xscale','log'); set(gca,'XTick',[.1 .2 .3 .4 .5 .6 .7 .8 .9 1 2 3 4 5 6 7 8 9 10 20]);
    set(gca,'XTickLabel','.1|.2|||.5|||||1|2||4||6||||10|20'); set(gca,'XMinorTick','off'); set(gca,'XMinorGrid','off')
    xlabel('Frequency [kHz]'); xlim([.2 11]);
    set(gca,'yscale','log');
    ylabel('P_k');
    title('Cavity Pressures');
    if exist('len4c','var')
        lgdstr = [num2str(10^-2*round(len*10^2).') repmat(' [mm], \Delta_{4C}=',4,1) num2str(10^-2*round(10^2*len4c)-10^-2*round(10^2*len).')];
        legend(lgdstr,'location','southwest')
    else % no existing calibration
        lgdstr = [num2str(10^-2*round(10^2*len).') repmat(' [mm]',4,1)];
        legend(lgdstr,'location','southwest')
    end
    filename=[location, 'CavityPressures'];
    set(gcf,'PaperPositionMode','auto')
    eval(char(strcat('print -depsc2 ',{' '},filename)))
    eval(char(strcat('print -dbitmap ',{' '},filename)))
    saveas(gcf,filename,'fig')
    
    if pauseFigs ==1, pause(5); end; close(h0)
end % if showFigs

% save workspace data
save([location,'findNullData.mat'])

disp(['beta0  :    ',num2str(beta0)])
disp(['L  [mm]:    ',num2str(len)])
disp(['D  [mm]:    ',num2str(dia)])
disp(['Error? :    ',num2str(err)])
if exist('len4c','var'),
    if size(len4c,1)>1, len4c = len4c.';end
    d4C = len-len4c;
    disp(['L-L4C[mm]:  ',num2str(d4C)])
end

end
