%Modified by JBA Jul23, to add return of etaprime. Couldnt find original version of MasonKappa.m

function [kappa,beta0,etaprime] = MasonKappa(f,D); %freq Hz D=dia in mm
%function [Zc,kappa,ro,c0] = LossyTubeParams(f,D)
%Formulae evaluated based on Mason 1927, p 259
if nargin==0
    f=1000;
    D=7.5;
    disp(sprintf('freq = %g [Hz], Dia = %g [mm]',f,D));
end

%f=200:20:4000;
%D=8; % [mm]

%Work in MKS units throughout
%f in Hz;
w = 2*pi*f; %radian frequency
%D in mm;
dia = D/1000; %convert diameters from [mm] to [m]

%Aux tube parameters
P = pi * dia; % Perimeter
S = pi * (dia/2)^2; % Area

%thermodynamics constants
P0 = 1e5; %1 atm
eta = 1.4; %cp/cf 
rho = 1.2; %density MKS
mu = 18.6e-6; %Viscosity [Pa-s] MKS  (Internet says 1.862e-5 SI Units)
K = mu*5/2; %coef of heat conductivity

 %Not used in this calculation
K1=25.4e-3 %thermal conductivity [N/sK] (Beranek & Mellow, 2012, p. 181)

c0=sqrt(eta*P0/rho);

%From Mason 1928, Physical Review Vol 31, Feb. pp 283-295
%etaprime = 6.618045e-3 (error: 2e-10)
etaprime = sqrt(mu) + sqrt(K) * ( sqrt(eta) - 1/sqrt(eta) );
etaprime = sqrt(mu)*(1+sqrt(5/2)*(sqrt(eta) - 1/sqrt(eta)));
etap=sqrt(mu)*( 1+sqrt(5/2)*( sqrt(eta)-1/sqrt(eta) ) );

beta0 = etaprime*2/(dia*sqrt(rho)); %Verified from MiddleEarWave pdf
%beta0= etaprime/R/sqrt(rho); %Assume cylindar R=2S/P

  fc= beta0^2/(2*pi); %cutoff freq where real and imag parts are equal

disp(sprintf('beta %g, beta^2/2pi = %g Hz ',beta0,fc));

%etaprime = 0.0065
%P/S = 500
%etaprime*P/S between 2pi and pi
Factor  = (etaprime*P/S)./sqrt(2*w*rho);
%Factor2 may be written in its analytic form: A good idea for clarity
%Verify: j=e^{j pi/2}; sqrt(j) = e^{j pi/4}; 1/sqrt(j) = e^{-j pi/4}

%Imag part of Factor 2 is small above 100 Hz, important below 100 Hz
Factor2 = 1 + Factor - j*Factor;  %Verify: sqrt(1/j) =e^(-j/2) = (1-j)/2 

%Eq 3 of Mason BSTJ, April 1927 pp 258-294 (imped.bib)
kappa_sq = -(w/c0).^2 .* Factor2;

kappa = sqrt(kappa_sq); %Helmholtz-Kirchoff complex propagation function

%convert back to the same units as keefe.m (1/mm)
kappa=kappa/1000;

%for now:
Zc = 0;
r0 = 0;
