%Program to determine the lenghts of two cavities, directly from the pressure
% And then compute the Norton parameters using a 2-cavitiy calibration
%
%Here are several TP files from MEEI data set. 
%One contains a 'perfect' probe calibration results. 
%The other four (a-d) were measured back to back on the same day with very similar control results
% - passed but not perfect.
%
% Remove the pole in the data using the admittance and the estimated lengths
%

%opengl software; %"software" is essential for the "real-time mouse"

set(0,'defaultaxeslinewidth',1)
set(0,'DefaultLineLineWidth',2)
%set(0,'defaultlinelineWidth',2)

set(0,'DefaultAxesFontSize',16);

addpath('./m');
%clear all;
close all
load ./Data/lastTPData_Perfect.mat; % off by 20um re best Dia of 8.2 mm
%load ./Data/lastTPData_ER10CHID_14A_20100217083823_aa.mat %0um
%load ./Data/lastTPData_ER10CHID_14A_20100217085654_bb.mat  %-30
%load ./Data/lastTPData_ER10CHID_14A_20100217090258_cc.mat  %-15
%load ./Data/lastTPData_ER10CHID_14A_20100217091727_dd.mat  %30um

y0=1/z0;
P4=Pl4ch1(:,4);
P3=Pl4ch1(:,3);
P2=Pl4ch1(:,2);
P1=Pl4ch1(:,1);

%Pats files from Mar 30
%load ./Data/lastTPData_9a_ER10CHID_14A_20130330144306.mat %10 um
%load ./Data/lastTPData_9b_ER10CHID_14A_20130330144418.mat %30 um

F=f; 

% %Generate a log scale with Nppo points per octave
kflog = LogIndex(F,2); %find F index for 2 pts/oct
%replace last two by hand
kflog(9)=157; kflog(10)=207;
Flog=F(kflog); %Integer freqs on log scale

s=i*2*pi*F; %compute along the j\omega axis
F=F/1e3;Flog=Flog/1e3; %Convert to [kHz]

%Loop over length and plot error
 %k=135:180; %limit freq range to zero of the pressure of cavity 4
 %k=120:190;
 %k=148:168;
 kfreq(4,:)=148:175;
 kfreq(3,:)=kfreq(4,:)+50;
 
fig(1);
loglog(F(kfreq(4,:)),abs(Pl4ch1(kfreq(4,:),4)));hold on
loglog(F,abs(Pl4ch1(:,4)),'--');hold on
loglog(F(kfreq(3,:)),abs(Pl4ch1(kfreq(3,:),3)),'r')
loglog(F,abs(Pl4ch1(:,3)),'r-.')
loglog(F,abs(Pl4ch1(:,2)),'g')
loglog(F(kflog),abs(Pl4ch1(kflog,2)),'go'); %<---
loglog(F,abs(Pl4ch1(:,1)),'c')
loglog(F(kflog),abs(Pl4ch1(kflog,1)),'co'); %<---
hold off; xlim([0.1 10]);grid on
title('Pressure data for Cavity 1-4')
xlabel('Freq [kHz]')
ylabel('\(Pressure_k(\omega)\)');

 print -depsc2 ./eps/4cav2C

%compare Nu and Du 
L1=TPlength(1); L2=TPlength(2); L3=TPlength(3); L4=TPlength(4); %Channel 1

%Comput normalized Y_3 and Y_4
[Zc,gamma,rc,c0]=LossyTubeParams(dia,f,TempC);% Dia [mm], f[kHz], T [C]
Gamma1=exp(-gamma*2*L1); Y1 = (1-Gamma1)./(1+Gamma1);
Gamma2=exp(-gamma*2*L2); Y2 = (1-Gamma2)./(1+Gamma2);
Gamma3=exp(-gamma*2*L3); Y3 = (1-Gamma3)./(1+Gamma3);
Gamma4=exp(-gamma*2*L4); Y4 = (1-Gamma4)./(1+Gamma4);

fig(2)
subplot(211);
UsNum = Y4 - Y3;
semilogy(F,abs(UsNum),'k', F(kflog),abs(UsNum(kflog)),'ks')
hold on;

UsDenom = 2*(1./P4-1./P3); % D_U 
semilogy(F,abs(UsDenom),'r',F(kflog),abs(UsDenom(kflog)),'ro')

xlim([3.5,5.5]);
ylim([0.5e1,2e2]);

ylabel('Pressure relationship');
title('\(ks: Y_4-Y_3; ro: [P_4^{-1} - P_3^{-1}]; b: P_{1,2,3}/P_{4}\)','FontSize',16);
grid on

%Now plot UsDenom again on a blown-up  scale
subplot(212);
UsHat = UsNum./UsDenom;

loglog(F,abs(UsHat),'k', F(kflog),abs(UsHat(kflog)),'kp');hold on;
%Now plot the 4cavity Us
Us=4.67e6*Us(:,1); %scale up for display
loglog(F,abs(Us),'r-.', F(kflog),abs(Us(kflog)),'rs');
hold off
grid on;
ylim([1e-1 2]);
title('ro: Us (4-cavity); kp: Us (Eq. 3)');
xlabel('Frequency [kHz]');
ylabel('\(U_s, 2\times2\) solution');

 print -depsc2 ./eps/PressureRelations
 

 fig(3);
 subplot(211);
 plot(F,20*log10(abs(Us./UsHat)));grid
 ylabel('|Us/UsHat| [dB]','FontSize',16);
 xlabel('Frequency [kHz]','FontSize',16);
 title('Error of \(U_s\), between the 4 and 2 cavity solutions [dB]','FontSize',16)
 
 print -depsc2 ./eps/UsHatErrorDB

 %Plot the nature of the errors in YkPk
 fig(4);
 Y3P3 = 1.8*Y3.*P3/10;
 Y4P4 = 1.8*Y4.*P4/10;
 
 subplot(211)
 loglog(F,abs(Y3P3),'k-',F,abs(Y4P4),'g-');
 hold on;
 
 YsDenom=1.8*2*(P4-P3)/5;
 loglog(F,abs(YsDenom),'b-',F(kflog),abs(YsDenom(kflog)),'bs' ); 
 %YsDenom=1.8*2*(P2-P1)/5;  %Cav 1,2 dont work as well
 %loglog(F,abs(YsDenom),'b-',F(kflog),abs(YsDenom(kflog)),'bo' ); 
  
 Ys2C = 1.8*0.3*(Y3.*P3 - Y4.*P4)./(P4-P3);  %This is where the branch-cut error is
 Ys4C = 1.8*0.3e7*Ys(:,1); 
  loglog(F,abs(Ys2C),'r-', F,abs(Ys4C),'m-');  %This is the r:2C vs m:4C comparison

 %Ys2Calt = (Y3./P4 - Y4./P3)./(1./P3 - 1./P4); 
 %loglog(F,abs(Ys2C),'b-', F,abs(Ys4C),'m-', F,abs(Ys2Calt),'g.-');
 title('\(bs,o: D_Y; k,g: Y_k P_k; N_Y: b: Y_s^{2C} m: Y_s^{4C};\)');
 ylabel('\(|Y_3 P_3|, |Y_4 P_4|\)');
 xlabel('Frequency');
 grid
 ylim([1e-2 1e0]);
 hold off;
 
 print -depsc2 ./eps/YsError
 
 %show the two velocity polar plots: Num of Ys2Calt line 138.
 fig(5)
 plot(Y3.*P3,'b'); hold on; plot(Y4.*P4,'m');
 klow=kflog(9);khi=kflog(10);%3.75 and 4.78 kHz
 U3low=Y3(klow).*P3(klow); U3hi=Y3(khi).*P3(khi);
 plot(U3low,'bo');plot(U3hi,'bp');%Plot 2 pts on polar plot
 U4low=Y4(klow).*P4(klow); U4hi=Y4(khi).*P4(khi);
 plot(U4low,'ro');plot(U4hi,'rp');%Plot 2 pts on polar plot
 k=klow:khi; %plot dasked line between peaks
 plot(Y3(k).*P3(k),'y--'); hold on; plot(Y4(k).*P4(k),'g--');

 title('Ploar plots of \(b: Y_3 P_3; \;\; m: Y_4 P_4\)');

 input('Hit any key to continue');

disp([' To estimate the relative lengths']) %: Polar Plots of P_k/P_4
fig(6); 
kf=1:85; ki=1:10;
dy=0.005;

subplot(212)
%Plot pressure ratios
%Change the plot to ratios of magnitudes
Y=zeros(length(kf),1);

P14=abs(P1(kf))./abs(P4(kf)); hold on;
plot(P14(ki),Y(ki),'ko');hold on;%plot(P14,Y,'k'); 
p14=mean(P14(ki)); text(real(p14), -dy+imag(p14),'P14');
plot(p14,-dy,'kp','markersize',10);

P24=abs(P2(kf))./abs(P4(kf));
plot(P24(ki),Y(ki),'ko'); %plot(P24,Y,'k'); 
p24=mean(P24(ki)); text(real(p24), -dy+imag(p24),'P24');
plot(p24,-dy,'kp','markersize',10)

P34=abs(P3(kf))./abs(P4(kf));
plot(P34(ki),Y(ki),'ko'); %plot(P34,Y,'k'); 
p34=mean(P34(ki)); text(real(p34)-1*dy,-dy+imag(p34),'P34');
plot(p34,-dy,'kp','MarkerSize',10);


%Plot impedance (i.e., admittance) ratios
%Z3/Z4 (i.e.: Y4/Y3) on the same pressure axis
Y43=Y4(kf)./Y3(kf);
plot(Y43(ki),'b-o');plot(Y43,'b-','LineWidth',2);
z43=mean(Y43(ki));
text(real(z43),-2*dy+imag(z43), 'Z_3/Z_4');
plot(abs(z43),-2*dy,'s','MarkerSize',10);

Y42=Y4(kf)./Y2(kf);
plot(Y42(ki),'b-o');plot(Y42,'b--','LineWidth',2);
z42=mean(Y42(ki));
text(real(z42),-2*dy+imag(z42), 'Z_2/Z_4');
plot(abs(z42),-2*dy,'s','MarkerSize',10);

Y41=Y4(kf)./Y1(kf);
plot(Y41(ki),'b-o');plot(Y41,'b-.','LineWidth',2);
z41=mean(Y4(ki)./Y1(ki));
text(real(z41), -2*dy+imag(z41), 'Z_1/Z_4');
plot(abs(z41),-2*dy,'s','MarkerSize',10);

%Regression line for reference
%xx=0:.25:3.5; yy= 0.18*(xx-1.1); zz=xx+j*yy; plot(zz,'k--','LineWidth',2)

%xlim([1 2.5]);ylim([-.02 .3]);

xlabel('\(\Re P_k/P_4\)', 'FontSize',16);
ylabel('\(\Im P_k/P_4\)', 'FontSize',16);
grid
%title(sprintf('k--:y=0.18(x-1.1); bo: Y_4/Y_3;  F_{min}: %1.2g, F_{max} %1.3g [kHz]',F(1),F(kf(end)) ), 'FontSize',16);
title(sprintf('\(P_{3,2,1}/P_4\);%2.3g, %2.3g, %2.3g; \(L_4/L_{3,2,1}\):%2.3g, %2.3g, %2.3g',p34,p24,p14,L4/L3, L4/L2, L4/L1 ), 'FontSize',16);
hold off


%Plot velocity presusre Nu and Du relationships
subplot(211);

plot(F,abs(P3./P4),'b-.'); %Pressure ratios
hold on
plot(F,abs(P2./P4),'b--')
plot(F,abs(P1./P4),'b')
hold off
ylim([1 3])
xlim([.1 2.1]);
grid
title('Ratio of cavity pressures');
ylabel('\(|P_k/P_4|\)');
xlabel('Frequency [kHz]')



print -depsc2 ./eps/LF-PresRatio

disp(['Next plot gives length norm'])  % = sum(abs(U_s)) as a function of cavity length'];

input('hit any key to continue');

%Colors for plotting
 c=['r-'; 'g-'; 'b-'; 'k-'; 'g-'; 'c-'; 'y-'; 'r-'; 'g-'; 'b-'; 'y-'; 'c-']; %colors
 c=[c;c;c];


%for icav=3:4;
icav=4; %Not sure how this loop works
   
%vary about nominal length
 Len4 = (-.02:.001:.02) + L4; %length [mm] of cavity 4(long) for channel 1
 Len3 = Len4 + L3-L4;
 
 Dia = (-2:.5:2) + dia; %vary about nominal diameter

  Error=zeros(length(Len4)-1,length(Dia));
 
 fig(7);
 AbsMin=1000;lmin=0;dmin=0;
 
 for d=1:length(Dia);
% Find Y_k(f)
%;gamma(f); rc=surge resistance
[Zc,gamma,rc,c0]=LossyTubeParams(Dia(d),f,TempC); %Cavity dia in mm;freq Hz, TempC;

for l=1:length(Len4) %Vary L4 (longer cavity)
Gamma4=exp(-gamma*2*Len4(l)); Y4 = (1-Gamma4)./(1+Gamma4);
Gamma3=exp(-gamma*2*Len3(l)); Y3 = (1-Gamma3)./(1+Gamma3);

UsDenom = 1./P4 - 1./P3 ;

Us= (Y4 + Y3)./UsDenom;

Error(l,d)=sum(abs(diff(Us)));

%loglog(F,abs(Us),c(l)); hold on

%ylim([]);
xlim([3.6 4.2]);

%plot C(i,j) for the minimum error
  if Error(l,d) < AbsMin, 
    UsMin=Us; 
    AbsMin1=AbsMin; 
    AbsMin=Error(l,d); lmin=l; dmin=d;
   plot(F,abs(Us),'k-','LineWidth',1);hold on
   xlim([3 4.5]);grid on
   
    pause(.1)
    
disp( sprintf('Cav %g, AbsMin=%g, at lmin= %g, dmin= %g',icav,Error(l,d),lmin,dmin) );
 end % Error
end
%plot(CMin,'r--','LineWidth',3);
%hold off;
%ylim([-2 2]);
xlabel('Frequency [kHz]');
ylabel('\(Delta_k \equiv |C_{k+1}-C_{k}|\)');
title(cdtFname);

 end  %Dia loop

%if icav == 3
 fig(8);
 mesh(Error');grid;
  
 fig(9);
 GrandMin=min(min(Error));
 contour(Error-GrandMin-.002);grid;
 xlabel('Dia')
 ylabel('length');
 title('Cavity 3');
%end

% if icav == 4
%  fig(10);
%  mesh(Error');grid;
%   
%  fig(7);
%  GrandMin=min(min(Error));
%  contour(Error-GrandMin-.002);grid;
%  xlabel('Dia')
%  ylabel('length');
%  title('Cavity 4');
% end
%  
%end %icav loop
