%Modified version of EvalPoleEst.m to search for length and radius of chosen cavity
%Version 1.01 (2016)
%1.00 May 21, 2016: verified that the length and radius may be recovered from the pressure null
%1.01 May 23: Fix definition of cavity impedance to use the standard formula not a pair of zeros 
%
%Allow different cavities to be evaluated (default 3)
%
clear all;
close all
clc;

%Program to evaluate each of the data sets:
% Here are several TP files from MEEI data set.  One contains a 'perfect'
% probe calibration results.  The other four (a-d) were measured back
% to back on the same day with very similar control results - passed
% but not perfect.  Must be very frustrating.  What is the difference
% (causes) between the perfect and not so perfect?  The problem region for
% not so perfect is in 1.5 to 5 kHz region.  We talked about steel wire.
% I think one of the major issues is with the ear tips.  We can compare
% a brand new tip vs. a repeatedly used tip vs. not well-glued tip.

set(0,'defaultaxeslinewidth',1);
set(0,'DefaultLineLineWidth',2);
set(0,'DefaultAxesFontSize',16);

%load ../Data/lastTPData_Perfect.mat
%load ../mat/4cdata_MA17E_best.mat
load 4cdata_MA17E_best.mat

    Fsamp=sig.SR;
    c0=343; %m/x
    NFT=sig.N;
    NF=sig.nf;
    Fmax=sig.Fmax;
    Fmin=sig.Fmin;
    F=sig.FrequenciesAll;
    F=F(:); %col vect
        
    Zs=sourceImpedance(:);
    Ps=sourcePressure(:);
        
    Us = Ps(2:end)./Zs(2:end); Us=[0; Us];

F=F/1e3; Fmax=Fmax/1e3; Fmin=Fmin/1e3; %convert to kHz

% =========== define s = j omega ==============================================
s=1j*2*pi*F; %complex freq on j*omega axis
% =========================================================

Pl=cavityResponse.'; %4 cavity pressures

%Crude starting estimates, to get started (expressed as a freq index)
IF1K=fix(median(find(round(10*F) == 10)));
%IF2K=fix(median(find(round(10*F) == 20)));
IF3K=fix(median(find(round(10*F) == 30)));
IF4K=fix(median(find(round(10*F) == 38)));
IF5K=fix(median(find(round(100*F) == 505)));
IF6K=fix(median(find(round(100*F) == 605)));
IF8K=fix(median(find(round(10*F) == 90)));
IF=IF1K:IF8K; %limit freq resp from 1-9 kHz

IC1=1; IC2=2; IC3=3; IC4=4; %cavity index numbers

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fig(1);  %Plots of various source pressures & normalizations
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Ps Thevenin Pressure
loglog(F(IF), 0.62*abs(Ps(IF)),'c'); grid; hold on;
loglog(F(IF), 2*0.62*abs(Ps(IF)./Pl(IF,IC1)),'c.-'); % Ps/P1

title('Cyan: -- 0.62 P_s, .- 2*0.62 P_s/P_1, Blue: P_1, Yellow: P_2, Green: P_3, Red: P_4; Blue .: /P_1; (Data: MA17E_best)');
%title('Green: P_s; Cyan: Cav1; Yellow: Cav2; Blue: Cav 3; Red: Cav 4');

%C1 Pressure: Cyan
loglog(F(IF),abs(Pl(IF,IC1)),'b'); grid on; hold on;

%C2 Pressure: Yellow
loglog(F(IF),abs(Pl(IF,IC2)),'y'); grid on; hold on;

%C3 with 5.0 kHz Null
loglog(F(IF),abs(Pl(IF,IC3)),'g');%Ch 3 pressure
loglog(F(IF5K),abs(Pl(IF5K,IC3)),'gd');

%C4 with 3.8 kHz null
loglog(F(IF),abs(Pl(IF,IC4)),'r');%Ch 3 pressure
loglog(F(IF4K),abs(Pl(IF4K,IC4)),'ro');%Ch 3 pressure

%Remove all cavity pressures normalized by Ps
%loglog(F(IF), abs(Pl(IF,IC4)./Ps(IF)),'g');
%loglog(F(IF4K),abs(Pl(IF4K,IC4)./Ps(IF4K)),'go');
%loglog(F(IF4K),abs(Pl(IF4K,IC3)./Ps(IF4K)),'gd'); %Ch3/Ps@4K
%loglog(F(IF),abs(Pl(IF,IC3)./Ps(IF)),'g');

loglog(F(IF),abs(Pl(IF,IC3)./Pl(IF,IC1)),'b.'); %C3/C1
loglog(F(IF),abs(Pl(IF,IC4)./Pl(IF,IC1)),'b.'); %Ch4/Ch1

hold off; grid on;
ylim([1e-2,10]);
ylabel('Pressure: P_s, P_k, P_k/P_1');
ylabel('Frequency [kHz]');

print -depsc2 Pressures
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fig(2);  %Replot as polar
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%IF=IF3K:IF6K;
IF=(IF5K-20):(IF5K+40); %Just look around null freq for C3

%plot((Ps(IF)),'g'); %Polar plot of P1/Ps
%plot((Pl(IF,IC3)),'b'); %Polar plot of P1/Ps
plot((Pl(IF,IC3)./Ps(IF)),'g.-'); %Polar plot of P3/Ps
hold on; grid on;
title('Green: C3/Ps; Yellow: Cav2; Blue: Cav 3/Ps; Red: Cav 4');
plot((Pl(IF5K,IC3)./Ps(IF5K)),'gd'); %Polar plot of P3/Ps
plot((Pl(IF,IC3)./Pl(IF,IC1)),'r.-'); %Polar plot of P3/P1
plot((Pl(IF5K,IC3)./Pl(IF5K,IC1)),'ro'); %Polar plot of P3/P1

%Polar plts of IC4 around null @ 3.8 kHz
IF=(IF4K-20):(IF4K+40); %Just look around null freq for C4

plot((Pl(IF,IC4)./Ps(IF)),'g.-'); %Polar plot of P4/Ps
plot((Pl(IF4K,IC4)./Ps(IF4K)),'go'); %Polar plot of P4/Ps
plot((Pl(IF,IC4)./Pl(IF,IC1)),'r.-'); %Polar plot of P4/P1
plot((Pl(IF4K,IC4)./Pl(IF4K,IC1)),'ko'); %Polar plot of P4/P1

%loglog(F,abs(Ps),'k-.'); hold on;
%loglog(F,abs(Us),'y.')


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fig(3)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

R0=7.5/2; %Cavity Diameter: 7.5 [mm]
R0=R0/1000; %[meters] (SI Units)

%===================================================================
%Compute Beta0:  doc.MiddleEarWaveModel.15/Rebuttal.pdf
%===================================================================
% Define thermodynamics constants for AIR
P0 = 1e5; %[Pa] (1 atm)
eta = 1.4; %cp/cv 
rho = 1.2; %Density [kg/m^3]
mu = 18.2e-6; %Viscosity [Pa-s] MKS 
K = mu*5/2; %coef of heat conductivity

% Derived constants:
% kappa(s) = [s + beta0 sqrt(s) ]/c0 %Allen ear canal paper
c0=sqrt(eta*P0/rho); %speed of sound
etaprime = sqrt(mu) + sqrt(K) * ( sqrt(eta) - 1/sqrt(eta) ); %Mason via Rayleigh via Kirchhoff?

%DECREASE BETA BY 26 dB (20x) (DONT KNOW WHY THIS IS NECESSARY)
% beta0=etaprime/R/sqrt(rho); %theory says this
 beta0 = 0.044*etaprime/R0*sqrt(rho); %Practice says this (Off by 20x)
 %beta0 = etaprime/R0*sqrt(rho); %Practice says this (Off by 20x)
 kappa = (s+beta0*sqrt(s))/c0;
 %kappa = kappa/1000; %Length in units of mm
 
 disp(sprintf('Beta_0/20 = %4.4g (Beta theory = %4.4g)',beta0, 20*beta0));

%% May 21, 2016
%sp is the complex pole freq from Kappa(s) (May 2015 notebook)
% Z(s) = (1+exp(-2LKappa(s))/(1-exp(-2LKappa(s))) =0  %normalized impedance 
% -2L Kappa(s_p)= \ln(-1) = \pmj pi
% s_p + beta_0 sqrt(s_p) = j 2pi c0/4L = j omega_z [since f_zero = c_0L

len=11; % in [mm]
Gamma=exp(-2*len*kappa);
plot(Gamma);
grid on
str=sprintf('\Gamma(\omega): Len=%g',len);
title(str);
xlabel('\Re \Gamma(\omega)');
ylabel('\Im\Gamma(\omega)');
print('-depsc2', 'Gamma');
pause(.1)


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fig(4)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%TO DO:
%Compute the line integral in the complex plane for null cancellation metric

% Method: Search over freq and compute response on j\omega axis
% Hold radius on known value of 7.5/2 [mm] (for now)

%IFCstart=IFC-20; IFCend = IFC+20;
%IF=[IFCstart:IFCend];

% Pick cavity to be analyized
%icav=input('Cavity # to evaluate (e.g., 2-4):')
icav=4; % Force cavity 4
str=sprintf('Evaluate cavity %d:',icav);
disp(str);

% Use same freq range as figure 2
if icav == 3
    IFC=IF5K;
else
%C4 around null @ 3.8 kHz
    IFC=IF4K;
end

%Tweek IFC
%IFC=IFC-3;  %rough tune frequency

%LenEst2= c0/(4*F(IFC)); %Freq in [kHz] -> Len in [mm]
Len4C = [8.501508686114859 11.511088241121172 16.874059935251204 22.288265565356397 ];
LenEst2 = Len4C(4)-0.13; %Fine-tune length

disp(sprintf('Initial Len Est C%d: %4.4g; Len4C: %4.4g [mm]',icav, LenEst2, Len4C(icav)));

Fc=F(IFC); %frequency estimate for the null

LenStart=LenEst2-0.1; %LenEst2 -2
LenEnd = LenEst2+0.1; %LenEst2 +1

%==============================================================
for len=[LenStart:0.02:LenEnd]; %search over length
%==============================================================

% Freq fp [kHz] is any real (pole) freq, not quantized to F(ifp).
% Fine tune the cavity zero frequency:
%len=len+.01; %Finetune the length estimate

if len == LenStart; %Display freq range of search
    disp( sprintf( 'Search Len %g - %g [mm]',LenStart, LenEnd));
end

%if len == LenEst2; %Display initial length est
%    disp( sprintf( 'Length %g [mm]', len ));
%end

%====================== DEFINE sp ==============================

%Compute impedance using the actual formula
Z = (1-exp(-2*len*kappa))./(1+exp(-2*len*kappa));

% normalize pressure in cavity 4 and 3, by cavity 1
% The purpose of this is to clean up the variations at low freqs, below the zero
% so that there are no local min in this ratio
 R12=Pl(:,icav)./Pl(:,IC1); %Pressure normalized by cavity 1 (See Fig(1))
 %R12=Pl(:,icav); %This works just as well, but the phase has a delay
 E=Z.*R12; % Remove the 1/pressure pole at the zero
 
 
% Polar plot
plot(E(IF),'b.-'); pause(.2);hold on
 
if len < LenEst2;
plot(E(IF),'b'); hold on;grid on
elseif len == LenEst2;
    plot(E(IF),'g'); 
    pause(2)
else  % len > LenEst2
    plot(E(IF),'r');
end

% %% plot magnitude and phase of E 
% subplot(211);
% loglog(F,abs(E),'r');hold on;grid on;
% %loglog(F,abs(Z));
% %loglog(F,abs(R12));
% xlim([3,4]);
% 
%  if len == LenEst2;
%      loglog(F,abs(E),'g--');
%      subplot(212);
%      semilogx(F,unwrap(angle(E)),'r');hold on;grid on;
%      disp( sprintf( 'Best Length %g [mm]', len ));
%      pause(2)
%  end
%  %
%  subplot(212);
%  semilogx(F,unwrap(angle(E)),'r');hold on;grid on;
%  xlim([3,4]);

pause(0.1)
end

% subplot(211);
% title('|j\omega - c_0\kappa(s_p)|');
% pause(3)
% ylim([0.707,1.414]);
% hold off
% subplot(212);title('angle/pi');
% ylim([-1-0.1,1.1]);
% hold off
 
