%Modified version of EvalPoleEst.m to search for length and radius of chosen cavity
%
%Allow different cavities to be evaluated (default 3)
%
%clear all;
close all
clc;

%Program to evaluate each of the data sets:
% Here are several TP files from MEEI data set.  One contains a 'perfect'
% probe calibration results.  The other four (a-d) were measured back
% to back on the same day with very similar control results - passed
% but not perfect.  Must be very frustrating.  What is the difference
% (causes) between the perfect and not so perfect?  The problem region for
% not so perfect is in 1.5 to 5 kHz region.  We talked about steel wire.
% I think one of the major issues is with the ear tips.  We can compare
% a brand new tip vs. a repeatedly used tip vs. not well-glued tip.

set(0,'defaultaxeslinewidth',1);
set(0,'DefaultLineLineWidth',2);
set(0,'DefaultAxesFontSize',16);

%load ../Data/lastTPData_Perfect.mat
%load ../mat/4cdata_MA17E_best.mat
load 4cdata_MA17E_best.mat

    Fsamp=sig.SR;
    c0=343; %m/x
    NFT=sig.N;
    NF=sig.nf;
    Fmax=sig.Fmax;
    Fmin=sig.Fmin;
    F=sig.FrequenciesAll;
    F=F(:); %col vect
        
    Zs=sourceImpedance(:);
    Ps=sourcePressure(:);
        
    Us = Ps(2:end)./Zs(2:end); Us=[0; Us];

F=F/1e3; Fmax=Fmax/1e3; Fmin=Fmin/1e3; %convert to kHz

Pl=cavityResponse.'; %4 cavity pressures

IF1K=fix(median(find(round(10*F) == 10)));
%IF2K=fix(median(find(round(10*F) == 20)));
IF3K=fix(median(find(round(10*F) == 30)));
IF4K=fix(median(find(round(10*F) == 38)));
IF5K=fix(median(find(round(100*F) == 505)));
IF6K=fix(median(find(round(100*F) == 605)));
IF8K=fix(median(find(round(10*F) == 90)));
IF=IF1K:IF8K; %limit freq resp from 1-9 kHz

IC1=1; IC2=2; IC3=3; IC4=4;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fig(1);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%plot(Us);grid on;title('Source Pressure: MA17E_best')

%Ps Theven Pressure
loglog(F(IF),abs(Ps(IF)),'g'); grid; hold on;
%loglog(F(IF5K),abs(Ps(IF5K)),'gd');

title('Green: Ps; Cyan: Cav1; Yellow: Cav2; Blue: Cav 3; Red: Cav 4');

%C1 Pressure: Cyan
loglog(F(IF),abs(Pl(IF,IC1)),'c'); grid on; hold on;

%C2 Pressure: Yellow
loglog(F(IF),abs(Pl(IF,IC2)),'y'); grid on; hold on;

%C3 with 5Kz Null
loglog(F(IF),abs(Pl(IF,IC3)),'b');%Ch 3 pressure
loglog(F(IF5K),abs(Pl(IF5K,IC3)),'bd');

loglog(F(IF),abs(Pl(IF,IC4)),'r');%Ch 3 pressure
loglog(F(IF4K),abs(Pl(IF4K,IC4)),'ro');%Ch 3 pressure

%C4 pressure/Ps 3.8K Null
loglog(F(IF), abs(Pl(IF,IC4)./Ps(IF)),'g');

%C3 with 5Kz Null over Ps
loglog(F(IF),abs(Pl(IF,IC3)./Ps(IF)),'g');

%Ch 3 over Ch1
loglog(F(IF),abs(Pl(IF,IC3)./Pl(IF,IC1)),'r--'); %C3/C1

loglog(F(IF4K),abs(Pl(IF4K,IC4)./Ps(IF4K)),'go');
loglog(F(IF4K),abs(Pl(IF4K,IC3)./Ps(IF4K)),'gd'); %Ch3/Ps@4K
loglog(F(IF),abs(Pl(IF,IC4)./Pl(IF,IC1)),'r-.'); %Ch4/Ch1

hold off; grid on;

%grid on;title('Source Pressure: MA17E_best')


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fig(2);  %Replot as polar
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%IF=IF3K:IF6K;
IF=(IF5K-20):(IF5K+40); %Just look around null freq for C3

%plot((Ps(IF)),'g'); %Polar plot of P1/Ps
%plot((Pl(IF,IC3)),'b'); %Polar plot of P1/Ps
plot((Pl(IF,IC3)./Ps(IF)),'g.-'); %Polar plot of P3/Ps
hold on; grid on;
title('Green: C3/Ps; Yellow: Cav2; Blue: Cav 3/Ps; Red: Cav 4');
plot((Pl(IF5K,IC3)./Ps(IF5K)),'gd'); %Polar plot of P3/Ps
plot((Pl(IF,IC3)./Pl(IF,IC1)),'r.-'); %Polar plot of P3/P1
plot((Pl(IF5K,IC3)./Pl(IF5K,IC1)),'ro'); %Polar plot of P3/P1

%Polar plts of IC4 around null @ 3.8 kHz
IF=(IF4K-20):(IF4K+40); %Just look around null freq for C4

plot((Pl(IF,IC4)./Ps(IF)),'g.-'); %Polar plot of P4/Ps
plot((Pl(IF4K,IC4)./Ps(IF4K)),'go'); %Polar plot of P4/Ps
plot((Pl(IF,IC4)./Pl(IF,IC1)),'r.-'); %Polar plot of P4/P1
plot((Pl(IF4K,IC4)./Pl(IF4K,IC1)),'ko'); %Polar plot of P4/P1

%loglog(F,abs(Ps),'k-.'); hold on;
%loglog(F,abs(Us),'y.')


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fig(3)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%icav=input('Cavity # to evaluate (e.g., 2-4):')

% Pick cavity to be analyized
icav=4;
str=sprintf('Evaluate cavity %d:',icav);
disp(str);

% Use same freq range as figure 2
if icav == 3
    IFC=IF5K;
else
%C4 around null @ 3.8 kHz
    IFC=IF4K;
end

s=1j*2*pi*F; %complex freq on j*omega axis

%Compute the line integral in the complex plane for null cancellation metric
%TO DO:
%Estimate the length from the zero frequency using f_c \lambda = c_0 with
% LenEst2C = \lambda/4 = c0/4*f_c assuming a rigid end.

LenEst2= c0/(4*F(IFC));

Len4C = [8.501508686114859 11.511088241121172 16.874059935251204 22.288265565356397 ];

disp(sprintf('Initial Len Est C%d: %4.4g; Len4C: %4.4g [mm]',icav, LenEst2, Len4C(icav)));
%disp(sprintf('Initial Len Est C4: %4.4g; Len4C: %4.4g [mm]', LenEst2C4, Len4C(4)));

Fc=F(IFC); %frequency estimate for the null
R0=7.5/2; %Cavity radius est [mm]
R0=R0/1000; %[meters]

%===================================================================
%Compute Beta0:  doc.MiddleEarWaveModel.15/Rebuttal.pdf
%===================================================================
% Define thermodynamics constants for AIR
P0 = 1e5; %1 atm
eta = 1.4; %cp/cf 
rho = 1.2; %density MKS
mu = 18.2e-6; %Viscosity [Pa-s] MKS 
K = mu*5/2; %coef of head conductivity

% Define basic constants
c0=sqrt(eta*P0/rho);
etaprime = sqrt(mu) + sqrt(K) * ( sqrt(eta) - 1/sqrt(eta) );
%beta0=etaprime/R/sqrt(rho); %theory says this
 beta0 = (0.05)*etaprime/R0*sqrt(rho); %Practice says this (Off by 20x)
 disp(sprintf('Beta_0 = %4.4g',beta0));
% kappa(s) = [s + beta0 sqrt(s) ]/c0 %Allen ear canal paper

%% OLD:
%s3=2*pi*(-0.022*1.999+0.9994*3.809i); %C3 Gustimate of pole rad-freq
%s4=2*pi*(-0.026+5039i); %C4 gustimate
%P3=((s+.026).^2)./( (s+s3).*( s+s3.' ) ); %Pole pair

%% May 21, 2016
%sp is the complex pole freq from Kappa(s) (May 2015 notebook)
% Z(s) = (1+exp(-2LKappa(s))/(1-exp(-2LKappa(s))) =0
% -2L Kappa(s_p)= \ln(-1) = \pmj pi
% s_p + beta_0 sqrt(s_p) = j 2pi c0/4L = j omega_z [since f_zero = c_0L

% Method: Search over freq and compute response on j\omega axis
% Hold radius on known value of 7.5/2 [mm] (for now)

%Tweek IFC
IFC=IFC-3

IFCstart=IFC-5; IFCend = IFC+5;
IF=[IFC-10 IFCstart:IFCend,IFC+10]
%==============================================================
for ifp=IF;     %Search over complex pole: IF is centered on IFC
%==============================================================

fp=F(ifp); %Search freq variable assumes beta=0

if ifp==IFCstart; %Display freq range of search
    disp( sprintf( 'Search Freq %g - %g [kHz]',F(IFCstart), F(IFCend) ));
end

if ifp==IFC; %Display center freq
    disp( sprintf( 'Center Freq %g [kHz]',F(IFC) ));
end

%s3=2*pi*(-df0*1.999+0.9994*3.809i); % Search over real part of s3
sp=1j*2*pi*fp; %Gustimate of imaginary pole freq (to search)

%INCREASE BETA BY 36 dB (50x), DONT KNOW WHY THIS IS NECESSARY
sp = sp + beta0*sqrt(sp); %complex pole freq = c0*Kappa(s)

ZP=(s-sp)./sp;             % 2d quadrant zero over positive freqs
ZM=(s-conj(sp))./conj(sp); % 3d quad zero over negative freqs

Z=ZP.*ZM; %combine the two zeros

 %[~,k3]=min(abs(imag(s-sp))); %Find index for sp
 %kd=50;
 %k=k3-kd:k3+kd;

%normalize pressure in cavity 4 and 3, by cavity 1
% The purpose of this is to clean up the variations at low freqs, below the zero
% so that there are no local min in this ratio
 R12=Pl(:,icav)./Pl(:,IC1); %Pressure normalized by cavity 1 (See Fig(1))
 R12=1./R12; %reciprocal normalize pressure
 E=R12.*conj(Z); % Remove the 1/pressure pole at the zero
 
 %% Polar plot
 if ifp < IFC
plot(E(IF),'b'); hold on;grid on
 elseif ifp == IFC;
     plot(E(IF),'g');
     pause(1)
 else  % ifp > IFC
     plot(E(IF),'r');
 end

% %% plot magnitude and phase of E 
% subplot(211)
% loglog( F(IF), abs(E(IF)), '.-' ); hold on; grid on
% %loglog(F(IF),5*abs(Pl(IF,icav)./Pl(IF,IC1)),'r'); %C3/C1
% %plot(E(IF),'k');grid on;hold on; %limit freq range for display only
% %plot(E(IFC),'mp','MarkerSize',15);
% 
% %plot(E(IFC),'og') %Positive freqs
% %plot(E(k(end-kd+1):k(end)),'sr'); %negative freqs
% subplot(212)
% semilogx( F(IF), unwrap( angle(E(IF)) )/pi ); hold on; grid on

pause(0.1)
end

% subplot(211);
% title('|j\omega - c_0\kappa(s_p)|');
% pause(3)
% ylim([0.707,1.414]);
% hold off
% subplot(212);title('angle/pi');
% ylim([-1-0.1,1.1]);
% hold off
 